


; Generates ions randomly on the filament
; within a radius r
; with random el and az angles
; Appelhans  February 2006



; definition of user adjustable variables  -----------------------

			 ; ion flight control parameters

defa _rerun 					0		; to turn rerun on (1) or off (0)

  ; ---------- adjustable at beginning of flight -----------------

			 ; ion randomization parameters

defa _el_Angle_Off_Vel_Axis  90.0	 ; (+- 90 deg) cone angle
defa _az_Angle_Off_Vel_Axis   90.0	  ; (+- 90 deg) cone angle
defa _x_Random_Offset_mm	   0.0		; random offset for x
defa _y_Random_Offset_mm	   0	 ; random offset for y
defa _z_Random_Offset_mm	   0	 ; random offset for z
defa _r_radius_mm				0.18	 ; radius of ion emission
defa _del_r_radius_mm			0.30	 ; width of ion emission annulus
defa  y_angle					 0.0
defa del_r_mm					0.0
defa z_2						0
defa y_2						0
defa factor 					0
; definition of static variables -----------------------------
									  ; trap flags and process parameters
defs first					   0.0	  ; first call flag
defs initial_energy 		   0.0	  ; initial energy of current ion
defs initial_energy_flag	   0.0	  ; compute initial energy of ion if 0


; program segments below --------------------------------------------

seg initialize		; randomize ion's position, and direction

			rcl _rerun
			sto rerun_flym				; turn rerun mode on or off
   ;	 1 sto Rerun_Flym				; force rerun on
										; turns traj file saving off
		;------------------- get ion's initial velocity components -------------
		;message ; hello
		rcl ion_vz_mm					; get ion's specified velocity components
		rcl ion_vy_mm
		rcl ion_vx_mm

		;------------------- convert to 3d polar coords -------------
		>p3d							; convert to polar 3d

		;------------------- save polar coord values ----------------
		sto speed rlup					; store ion's speed
		sto az_angle rlup				; store ion's az angle
		sto el_angle					; store ion's el angle


		;------------------- make sure el_Angle_Off_Vel_Axis is legal -------------
								; force 0 <= el_Angle_Off_Vel_Axis <= 180
		rcl _el_Angle_Off_Vel_Axis abs
		180 x>y rlup sto _el_Angle_Off_Vel_Axis


		;-- compute randomized el angle

		2 rcl _el_Angle_Off_Vel_Axis * rand *
		rcl _el_Angle_Off_Vel_Axis - ;

		;-- compute randomized az angle

		2 rcl _az_Angle_Off_Vel_Axis * rand *
		rcl _az_Angle_Off_Vel_Axis - 90 +


		;---------------------- recall new ion speed ------------------
		rcl speed						; recall new speed

		;--------- at this point x = speed, y = az, z = el --------------
		;------------- convert to rectangular velocity components ---------
		>r3d							; convert polar 3d to rect 3d

		;------------- el rotate back to from 90 vertical -------------
		-90 >elr

		;------------- el rotate back to starting elevation -------------
		rcl el_angle >elr

		;------------- az rotate back to starting azimuth -------------
		rcl az_angle >azr

		;------------- update ion's velocity components with new values --------
		sto ion_vx_mm					; return vx
		rlup
		sto ion_vy_mm					; return vy
		rlup
		sto ion_vz_mm					; return vz

 ;--------- randomize ion's position components --------

		; x component (not part of circle)

		rcl _x_Random_Offset_mm
		2 / sto x_half_pos				   ; save half max shift

		rcl ion_px_mm					   ; get nominal x start
		rcl _x_Random_Offset_mm rand * +   ; add random shift
		rcl x_half_pos -				   ; subtract half shift
		sto ion_px_mm					   ; store random x start



	;  following is for an annular beam at random angles with a set annular width

	 rand 0.5 - rcl _del_r_radius_mm *	 ; width of ion emission annulus
	 sto  del_r_mm

	 rand								  ; get random number
	 6.2832 * sto y_angle				  ; compute random angle (radians)
	 sin rcl _r_radius_mm rcl del_r_mm + * ; compute y distance with offset
	 sto _y_random_offset_mm
	 rcl ion_py_mm +
	 sto ion_py_mm						  ; save to ion position

	 6.2832 rcl y_angle -				  ; compute random angle (radians)
	 cos rcl _r_radius_mm rcl del_r_mm + * ; compute z distance with offset
	 sto _z_random_offset_mm
	 rcl ion_pz_mm +
	 sto ion_pz_mm						  ; save to ion position


	; following is for recitilinear random ion position start
	; modified to limit starting region to a circular area
	;
  ;    lbl loop 							  ; randomize radius

  ; 		 rand 0.5 - 						; allows for +/- about center
  ; 		 sto y_half_pos 					; save half max shift
  ; 		 rcl y_half_pos entr *
  ; 		 sto y_2							; y squared
  ;
  ; 		 rand 0.5 - 						; allows for +/- about center
  ; 		 sto z_half_pos 					; save half max shift
  ; 		 rcl z_half_pos entr *
  ; 		 sto z_2							; z squared

  ; 		 rcl y_2
  ; 		 + sqrt 							; calculate length of vector
  ; 		 2 *								; double to correct for original halving
  ; 		 sto factor
  ; 		 rcl _r_radius_mm * 				; check that vector is less that desired length
  ; 		 rcl _r_radius_mm
  ; 		 x<y gto loop						; if not, try again
  ;
  ; 		 rcl ion_py_mm						; get nominal y start
  ; 		 rcl y_half_pos rcl factor *		; adjust length  by factor
  ; 		 rcl _r_radius_mm * 2 * +				; scale to desired radius, add to starting position
  ; 		 sto ion_py_mm						; store random y start
  ;
  ; 		 rcl ion_pz_mm						; get nominal z start
  ; 		 rcl z_half_pos rcl factor *		; adjust length  by factor
  ; 		 rcl _r_radius_mm * 2 * +				; scale to desired radius, add to starting position
  ; 		 sto ion_pz_mm						; store random z start
  ;

		 exit

		;---------------------- done -----------------------------------------


seg Terminate
;			 rcl _rerun
;			 sto rerun_flym 			   ; reset rerun mode
